# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from setuptools import setup

setup(
    name='module_to_install_in_editable_mode',
    packages=[],
)
